<?php

/**
 * @file
 * Admin page callbacks for the archibald_comments module.
 */

if (!defined("ARCHIBALD_COMMENT_NOT_PUBLISHED")) {
  // Comment is awaiting approval.
  define('ARCHIBALD_COMMENT_NOT_PUBLISHED', 0);
}

if (!defined("ARCHIBALD_COMMENT_PUBLISHED")) {
  // Comment is published.
  define('ARCHIBALD_COMMENT_PUBLISHED', 1);
}

if (!defined("ARCHIBALD_COMMENT_DELETED")) {
  // Comment is deleted.
  define('ARCHIBALD_COMMENT_DELETED', 2);
}

/**
 * Menu callback; present an administrative comment listing.
 */
function archibald_comments_admin($type = 'new') {
  $edit = $_POST;

  if (isset($edit['operation']) && ($edit['operation'] == 'delete') && isset($edit['comments']) && $edit['comments']) {
    return drupal_get_form('archibald_comments_multiple_delete_confirm');
  }
  else {
    return drupal_get_form('archibald_comments_admin_overview', $type);
  }
}

/**
 * Form builder for the comment overview administration form.
 *
 * @param $arg
 *   Current path's fourth component: the type of overview form ('approval' or
 *   'new').
 *
 * @ingroup forms
 * @see archibald_comments_admin_overview_validate()
 * @see archibald_comments_admin_overview_submit()
 */
function archibald_comments_admin_overview($form, &$form_state, $arg) {
  // Build an 'Update options' form.
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => '',
    '#attributes' => array('class' => array('container-inline')),
  );

  $status = NULL;
  if ($arg == 'approval') {
    $status = ARCHIBALD_COMMENT_NOT_PUBLISHED;
    $options['publish'] = t('Publish the selected comments');
  }
  else {
    $status = ARCHIBALD_COMMENT_PUBLISHED;
    $options['unpublish'] = t('Unpublish the selected comments');
  }
  $options['delete'] = t('Delete the selected comments');

  // Check if user has put a search term
  if (isset($form_state['input']['search_field'])) {
    if (empty($form_state['input']['search_field'])) {
      unset($_SESSION['comment_search_field']);
    }
    else {
      $_SESSION['comment_search_field'] = $form_state['input']['search_field'];
    }
  }

  $comment_search_field = NULL;
  if (isset($_SESSION['comment_search_field'])) {
    $comment_search_field = $_SESSION['comment_search_field'];
  }

  $form['options']['operation'] = array(
    '#type' => 'select',
    '#title' => t('Update options'),
    '#title_display' => 'before',
    '#options' => $options,
    '#default_value' => 'publish',
  );
  $form['options']['submit_update'] = array(
    '#type' => 'submit',
    '#name' => 'update',
    '#value' => t('Update'),
  );

  // Add a search textfield and search button
  $form['options']['search_field'] = array(
    '#type' => 'textfield',
    '#title' => t('Search'),
    '#title_display' => 'before',
    '#size' => 40,
    '#default_value' => $comment_search_field,
  );

  $form['options']['submit_search'] = array(
    '#type' => 'submit',
    '#name' => 'search',
    '#value' => t('Search'),
  );

  $header = array(
    'comment' => array(
      'data' => t('Comment'),
      'field' => 'comment',
    ),
    'author' => array(
      'data' => t('Author'),
      'field' => 'name',
    ),
    'posted_in' => array(
      'data' => t('Posted in'),
      'field' => 'term',
    ),
    'created' => array(
      'data' => t('Created'),
      'field' => 'c.created',
      'sort' => 'desc',
    ),
    'operations' => array(
      'data' => t('Operations'),
    ),
  );

  $query = db_select('archibald_comments', 'c')->extend('PagerDefault')->extend('TableSort');

  $query->join('archibald_lom', 'l', 'l.lom_id = c.lom_id');
  $query->join('archibald_general', 'g', 'l.general_id = g.general_id');
  $query->join('archibald_langstring_terms', 't', 'g.title = t.langstring_terms_id');
  $query->leftJoin('users', 'u', 'u.uid = c.uid');
  $query->groupBy('c.cid');

  $query->addField('u', 'name', 'u_name');
  $query->addField('c', 'subject', 'subject');
  $query->addField('c', 'comment', 'comment');
  $query->addField('c', 'created', 'created');

  $query->addField('c', 'uid', 'uid');
  $query->addField('c', 'name', 'c_name');
  $query->addField('c', 'mail', 'mail');
  $query->addField('c', 'cid', 'cid');
  $query->addField('c', 'lom_id', 'lom_id');

  $query->addField('t', 'term', 'lom_title');

  if ($comment_search_field) {
    $query->condition(
      db_or()
      ->condition('c.comment', '%' . $comment_search_field . '%', 'LIKE')
      ->condition('t.term', '%' . $comment_search_field . '%', 'LIKE')
      ->condition('c.name', '%' . $comment_search_field . '%', 'LIKE')
      ->condition('u.name', '%' . $comment_search_field . '%', 'LIKE')
    );
  }

  $comments = $query->fields('c', array('cid', 'subject', 'name', 'created'))->condition('c.status', $status)->limit(25)->orderByHeader($header)->execute();

  // Build a table listing the appropriate comments.
  $options = array();
  $destination = drupal_get_destination();



  while ($comment = $comments->fetchObject()) {
    $comment_dots = NULL;

    $options[$comment->cid] = array(
      'comment' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => truncate_utf8(
            $comment->comment,
            50,
            TRUE,
            TRUE
          ) . $comment_dots,
          '#href' => 'admin/archibald/comment/' . $comment->cid . '/edit',
          '#options' => array(
            'attributes' => array('title' => $comment->comment),
            'fragment' => 'comment-' . $comment->cid,
          ),
        ),
      ),
      'author' => archibald_comments_username($comment),
      'posted_in' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => $comment->lom_title,
          '#href' => 'archibald/' . $comment->lom_id,
        ),
      ),
      'created' => format_date($comment->created, 'short'),
      'operations' => array(
        'data' => array(
          '#type' => 'link',
          '#title' => t('edit'),
          '#href' => 'admin/archibald/comment/' . $comment->cid . '/edit',
          '#options' => array('query' => $destination),
        ),
      ),
    );
  }

  $form['comments'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $options,
    '#empty' => t('No comments available.'),
  );

  $form['pager'] = array('#theme' => 'pager');

  return $form;
}

/**
 * Validate archibald_comments_admin_overview form submissions.
 *
 * In case Update-Button is clicked, check if at leat one comment was selected.
 *
 * @param array $form
 * @param array $form_state
 */
function archibald_comments_admin_overview_validate($form, &$form_state) {
  if ($form_state['clicked_button']['#name'] == 'update') {
    $form_state['values']['comments'] = array_diff(
      $form_state['values']['comments'],
      array(0)
    );
    // We can't execute any 'Update options' if no comments were selected.
    if (count($form_state['values']['comments']) == 0) {
      form_set_error(
        '',
        t('Select one or more comments to perform the update on.')
      );
    }
  }
}

/**
 * Process archibald_comments_admin_overview form submissions.
 */
function archibald_comments_admin_overview_submit($form, &$form_state) {

  if ($form_state['clicked_button']['#name'] == 'update') {
    $operation = $form_state['values']['operation'];
    $cids = $form_state['values']['comments'];

    if ($operation == 'delete') {
      archibald_comments_delete($cids);
    }
    else {
      $status = NULL;
      if ($operation == 'unpublish') {
        $status = ARCHIBALD_COMMENT_NOT_PUBLISHED;
      }
      elseif ($operation == 'publish') {
        $status = ARCHIBALD_COMMENT_PUBLISHED;
      }
      archibald_comments_set_status($cids, $status);
    }
    drupal_set_message(t('The update has been performed.'));
    cache_clear_all();
  }
}

/**
 * List the selected comments and verify that the admin wants to delete them.
 *
 * @param $form_state
 *   An associative array containing the current state of the form.
 *
 * @return
 *   TRUE if the comments should be deleted, FALSE otherwise.
 * @ingroup forms
 * @see comment_multiple_delete_confirm_submit()
 */
function archibald_comments_multiple_delete_confirm($form, &$form_state) {
  $edit = $form_state['input'];

  $form['comments'] = array(
    '#prefix' => '<ul>',
    '#suffix' => '</ul>',
    '#tree' => TRUE,
  );

  $comment_counter = 0;

  $cids = array_filter($edit['comments']);

  $query = db_select('archibald_comments', 'c');
  $query->addField('c', 'comment', 'comment');
  $query->addField('c', 'cid', 'cid');
  $query->condition('cid', $cids, 'IN');
  $result = $query->execute();

  foreach ($result as $comment) {
    //$subject = db_query('SELECT subject FROM {archibald_comments}
    //WHERE cid = :cid', array(':cid' => $cid))->fetchField();
    $form['comments'][$comment->cid] = array(
      '#type' => 'hidden',
      '#value' => $comment->cid,
      '#prefix' => '<li>',
      '#suffix' => truncate_utf8(
        check_plain($comment->comment),
        100,
        TRUE,
        TRUE
      ) . '</li>',
    );
    $comment_counter++;
  }
  $form['operation'] = array(
    '#type' => 'hidden',
    '#value' => 'delete',
  );

  if (!$comment_counter) {
    drupal_set_message(
      t('There do not appear to be any comments to delete, or your selected comment was deleted by another administrator.')
    );
    drupal_goto('admin/archibald/comments');
  }
  else {
    return confirm_form(
      $form,
      t('Are you sure you want to delete these comments and all their children?'),
      'admin/archibald/comments',
      t('This action cannot be undone.'),
      t('Delete comments'),
      t('Cancel')
    );
  }
}

/**
 * Process comment_multiple_delete_confirm form submissions.
 */
function archibald_comments_multiple_delete_confirm_submit(
  $form,
  &$form_state
) {

  if ($form_state['values']['confirm']) {
    archibald_comments_delete(array_keys($form_state['values']['comments']));
    cache_clear_all();
    $count = count($form_state['values']['comments']);
    watchdog('content', 'Deleted @count comments.', array('@count' => $count));
    drupal_set_message(
      format_plural(
        $count,
        'Deleted 1 comment.',
        'Deleted @count comments.'
      )
    );
  }
  $form_state['redirect'] = 'admin/archibald/comments';
}

/**
 * Returns a menu title which includes the number of unapproved comments.
 *
 * @return string menu title
 */
function archibald_comments_count_unpublished() {
  $count = db_query(
    'SELECT COUNT(cid) FROM {archibald_comments} WHERE status = :status',
    array(
      ':status' => 0,
    )
  )->fetchField();
  return t('Unapproved comments (@count)', array('@count' => $count));
}

/**
 * Helper function:
 * Reads username from comment. In case uid is given, return username as link.
 * In case uid is not given (anonymous user),
 * return username with a hint "not verified".
 *
 * @param array $comment
 *
 * @return string username
 */
function archibald_comments_username($comment) {

  $return = NULL;

  if (!empty($comment->u_name)) {
    $return = array('data' =>
      array(
        '#type' => 'link',
        '#title' => $comment->u_name,
        '#href' => 'user/' . $comment->uid,
      ),
    );
  }
  elseif (!empty($comment->c_name)) {
    $return = $comment->c_name . ' (' . t('Not verified') . ')';
  }
  else {
    $return = t('Unknown person');
  }

  return $return;
}

/**
 * Helper function:
 * Write status flag for all $cids to $status into database.
 *
 * @param array $cids
 * @param int $status
 */
function archibald_comments_set_status(
  $cids,
  $status = ARCHIBALD_COMMENT_PUBLISHED
) {

  if (!empty($cids)) {
    $query = db_update('archibald_comments');
    $query->fields(array('status' => $status));
    $query->condition('cid', $cids, 'IN');
    if ($query->execute() > 0) {
      module_invoke('archibald', 'comment_updated', $cids, $status);
    }

    if ($status == ARCHIBALD_COMMENT_NOT_PUBLISHED) {
      module_load_include(
        'php',
        'archibald_comments',
        'includes/ArchibaldCommentsPublish.class'
      );

      $comments_publish = new ArchibaldCommentsPublish();
      foreach ($cids as $cid) {
        $comments_publish->delete($cid);
      }
    }
  }
}

/**
 * Helper function:
 * Delete all $cids from database.
 *
 * @param array
 *   $cids
 */
function archibald_comments_delete($cids) {
  if (!empty($cids)) {
    $query = db_delete('archibald_comments');
    $query->condition('cid', $cids, 'IN');
    $query->execute();
    module_invoke('archibald', 'comment_updated', $cids, ARCHIBALD_COMMENT_DELETED);
    module_load_include('php', 'archibald_comments', 'includes/CommentsPublish.class');

    $comments_publish = new ArchibaldCommentsPublish();
    foreach ($cids as $cid) {
      $comments_publish->delete($cid);
    }
  }
}

/**
 * Call drupal_set_title and drupal_get_form.
 *
 * @param int $cid
 *
 * @return array
 *   result of drupal_get_form
 */
function archibald_comments_edit_page($cid) {
  drupal_set_title(t('Edit Archibald comment'));
  return drupal_get_form("archibald_comments_comment_form", $cid);
}

/**
 * Creates form to define settings for the archibald_comments module.
 *
 * @return array form
 */
function archibald_comments_settings() {
  $form = array();

  $form['archibald_comments'] = array(
    '#type' => 'fieldset',
    '#title' => t('E-Mail notification for administrators'),
    '#description' => '',
    '#collapsible' => TRUE,
  );

  $form['archibald_comments']['email'] = array(
    '#type' => 'textfield',
    '#title' => t('Administrator E-Mail'),
    '#default_value' => variable_get('archibald_comments_admin_email', ''),
    '#description' => t('Send notifications to this E-Mail address. Leave blank to send no notifications.'),
  );

  $interval_options = array(
    0 => 'Each time a comment is submitted',
    3600 => 'Once an hour',
    3600 * 24 => 'Once a day',
    3600 * 24 * 7 => 'Once a week',
  );

  $default_interval = variable_get(
    'archibald_comments_admin_email_interval',
    0
  );

  $form['archibald_comments']['interval'] = array(
    '#type' => 'select',
    '#options' => $interval_options,
    '#title' => t('Notification interval'),
    '#default_value' => $default_interval,
    '#description' => t('Select time interval for E-Mail notification.'),
  );


  $form['archibald_comments_display'] = array(
    '#type' => 'fieldset',
    '#title' => t('Display options for comments'),
    '#description' => '',
    '#collapsible' => TRUE,
  );

  $comments_per_page_options = array(
    3 => 3,
    4 => 4,
    5 => 5,
    6 => 6,
    7 => 7,
    8 => 8,
    9 => 9,
    10 => 10,
    11 => 11,
    12 => 12,
    13 => 13,
    14 => 14,
    15 => 15,
  );

  $default_interval = variable_get('archibald_comments_per_page', 8);
  $form['archibald_comments_display']['number_per_page'] = array(
    '#type' => 'select',
    '#options' => $comments_per_page_options,
    '#title' => t('Number of comments displayed per page'),
    '#default_value' => $default_interval,
    '#description' => t('Select number of comments displayed per page.'),
  );


  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

/**
 * Validates archibald_comments settings form.
 * Checks if given E-Mail address is valid.
 *
 * @param array $form
 *   Nested array of form elements that comprise the form.
 * @param array $form_state
 *   A keyed array containing the current state of the form. The arguments
 *   that drupal_get_form() was originally called with are available in the
 *   array $form_state['build_info']['args'].
 *
 * @return bool
 */
function archibald_comments_settings_validate($form, &$form_state) {

  if (!empty($form_state['values']['email'])) {
    if (!valid_email_address($form_state['values']['email'])) {
      form_set_error('email', t('The email address appears to be invalid.'));
    }
  }
}

/**
 * Sets submitted settings values to variable table (variable_set).
 *
 * @param array $form
 *   Nested array of form elements that comprise the form.
 * @param array $form_state
 *   A keyed array containing the current state of the form. The arguments
 *   that drupal_get_form() was originally called with are available in the
 *   array $form_state['build_info']['args'].
 */
function archibald_comments_settings_submit($form, &$form_state) {
  variable_set(
    'archibald_comments_admin_email',
    $form_state['values']['email']
  );

  variable_set(
    'archibald_comments_admin_email_interval',
    $form_state['values']['interval']
  );

  variable_set(
    'archibald_comments_per_page',
    $form_state['values']['number_per_page']
  );
}

