<?php

/**
 * @file
 * comments publish handler, to submit local comment into the central catalog
 */

/**
 * Require abstract parent class
 */
module_load_include(
  'php',
  'archibald',
  'includes/dsb_api.class'
);

/**
 * Tis class publish comments of lom objetecs
 * where lom object is or was published
 *
 * @author Heiko Henning
 */
class ArchibaldCommentsPublish extends ArchibaldAbstractDsbApi {

  /**
   * here we reproduce the global lom id
   * see ArchibaldLomPublish::completeLomObject()  -> addIdentifier
   * @var string
   */
  protected $lomIdPrefix = 'educa###';

  /**
   * process all lom objects with are marked as to publish in database
   *
   * @return boolean
   *   status
   */
  public function cron() {
    // if the cron was stopped cause of bigger probles will processing
    if (variable_get('archibald_upload_error', 0) == 1) {
      if (function_exists('drush_log')) {
        drush_log(
          'Comments Publish: Upload error bit setted',
          'error'
        );
      }
      return FALSE;
    }

    $maintenance_max_execution_time = ini_get('max_execution_time');
    if ($maintenance_max_execution_time == 0) {
      $maintenance_max_execution_time = 240;
    }

    if ($maintenance_max_execution_time > 20) {
      $maintenance_max_execution_time -= 10;
    }

    if (!lock_acquire(
        'comments_publish',
        ($maintenance_max_execution_time + 1)
      )) {

      if (function_exists('drush_log')) {
        drush_log('Comments Publish: action is locked, perhaps a gui publish process is open', 'error');
      }
      return FALSE;
    }

    $processed_comments = 0;
    $maintenance_start_time = time();
    $res = db_query(
      "SELECT DISTINCT ls.lom_id, c.cid, c.uid, c.created, c.name, c.hostname, c.subject, c.comment " .
      "FROM {archibald_lom_stats} AS ls " .
      "INNER JOIN {archibald_lom} AS l ON (ls.lom_id=l.lom_id) " .
      "LEFT JOIN {archibald_comments} AS c on (c.lom_id=l.lom_id) " .
      "WHERE l.deleted IS NULL " .
      "AND ls.publication_progress=100 " .
      "AND c.status=:status " .
      "AND c.ident_dsb='' ",
      array(':status' => ARCHIBALD_COMMENT_PUBLISHED)
    );

    while ($comment = $res->fetchObject()) {
      // processingtime exceeded
      if (($maintenance_start_time + ($maintenance_max_execution_time / 2)) <=
        time()
      ) {
        break;
      }

      // on upload attempt we run into a fatal permanent error
      if (variable_get('archibald_upload_error') == 1) {
        break;
      }

      // init content partner settings
      $this->loadContentPartnerByLomId($lom_id);

      if (!$this->contentPartnerCanCentral()) {
        // this content partner is not allowed to access central catalog
        continue;
      }

      $result = $this->commentAdd(
        $comment->lom_id,
        $comment->name,
        $comment->uid,
        $comment->comment,
        $comment->created
      );

      if (!empty($result)) {
        if (function_exists('drush_log')) {
          drush_log(
            'Comments Publish: sucessfull ' . $comment->cid,
            'ok'
          );
        }
        db_query(
          'UPDATE {archibald_comments} SET ident_dsb=:ident_dsb WHERE cid=:cid',
          array(':ident_dsb' => $result, ':cid' => $comment->cid)
        );
      }
      else {
        db_query(
          'UPDATE {archibald_comments} SET ident_dsb=:ident_dsb WHERE cid=:cid',
          array(':ident_dsb' => 'error', ':cid' => $comment->cid)
        );
      }

      $processed_comments++;
    }

    if ($processed_comments == 0) {
      if (function_exists('drush_log')) {
        drush_log(
          'Comments Publish: nothing to do',
          'ok'
        );
      }
    }

    lock_release('comments_publish');
    return TRUE;
  }

  /**
   * delete comment by {archibald_comments}.cid
   *
   * @param integer $cid
   *
   * @return boolean
   *   status
   */
  public function delete($cid) {
    $res = db_query(
      'SELECT DISTINCT c.lom_id, c.cid, c.ident_dsb FROM {archibald_comments} c WHERE c.cid=:cid ',
      array(':cid' => $cid)
    );

    $comment = $res->fetchObject();
    if (!empty($comment->ident_dsb) && $comment->ident_dsb != 'error') {

      // init content partner settings
      $this->loadContentPartnerByLomId($lom_id);

      if (!$this->contentPartnerCanCentral()) {
        // this content partner is not allowed to access central catalog
        return FALSE;
      }

      $this->commentDelete($comment->ident_dsb, $comment->lom_id);
      db_query(
        "UPDATE {archibald_comments} SET ident_dsb='' WHERE cid=:cid ",
        array(':cid' => $cid)
      );
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Add comment to lom resource
   *
   * @param string $lom_id
   *   Lom id from resource_search or resoruce_get
   * @param string $author
   *   first and lastname of author
   * @param string $author_id
   *   string with user identification. Optional.
   *   Your local numeric id, sessionid or ip (to let user delete his own post)
   * @param string $text
   *   comment text, no html, no urls and no bbcode
   * @param integer $time
   *   default:NOW()  set the comment add time, format: unix timestamp
   * @param boolean $verbose
   *   default:TRUE
   *   set to FALSE to only return error messages and dont call execptions
   *
   * @return mixed
   *   status, if success return TRUE, else return the message
   */
  public function commentAdd($lom_id, $author, $author_id, $text, $time = 0, $verbose = TRUE) {
    // init content partner settings
    $this->loadContentPartnerByLomId($lom_id);

    if (!$this->contentPartnerCanCentral()) {
      return FALSE;
    }

    // init lom api
    $this->lomApiAuth($auth_token);

    $postarr = array(
      'AuthToken' => $auth_token,
      // The Lom ID
      'lom_id' => $this->lomIdPrefix . $lom_id,
      // The Author
      'author' => $author,
      // The Author identification IP
      'authorid' => $author_id,
      // The text
      'text' => $text,
    );

    if (!empty($time)) {
      $postarr['time'] = $time;
    }

    $raw_result = $this->openUrl(
      variable_get(
        'archibald_api_url',
        'https://main-api-dsb.educa.ch'
      ) . '/comment_add',
      $postarr,
      TRUE
    );

    $result = json_decode($raw_result);
    if (empty($result)) {
      if (function_exists('drush_log')) {
        drush_log(
          'comment add error: no parsable return: ' . $raw_result,
          'error'
        );

        watchdog(
          'comment add',
          'comment add error: can not parse result: !raw_result',
          array('!raw_result' => $raw_result),
          WATCHDOG_ERROR,
          'archibald/' . $lom_id
        );
      }
      return FALSE;
    }

    // session time out
    if ($result->code == 502) {
      $this->lomApiAuth($auth_token, TRUE);
      $postarr['AuthToken'] = $auth_token;
      $result = json_decode($this->openUrl(
          variable_get(
            'archibald_api_url',
            'https://main-api-dsb.educa.ch'
          ) . '/comment_add',
          $postarr,
          TRUE
        )
      );
    }

    if ($result->code != 200) {
      if (function_exists('drush_log') && $verbose == TRUE) {
        drush_log(
          'comment add error: ' . $result->code .
          ' (' . $result->message . ')',
          'error'
        );

        watchdog(
          'comment add',
          'comment add error: @code (!message)',
          array(
            '@code' => $result->code,
            '!message' => $result->message,
          ),
          WATCHDOG_ERROR,
          'archibald/' . $lom_id
        );
      }
      return FALSE;
    }
    else {
      return $result->data->commnet_id;
    }
  }

  /**
   * delete comment
   *
   * @param string $comment_id
   *  The comment id
   * @param string $lom_id
   *  The lom id
   * @param string $authorid
   *  the author id (optional, default empty)
   *
   * @return boolean
   *  status
   */
  public function commentDelete($comment_id, $lom_id, $authorid = '') {
    // init content partner settings
    $this->loadContentPartnerByLomId($lom_id);

    if (!$this->contentPartnerCanCentral()) {
      // this content partner is not allowed to access central catalog
      return FALSE;
    }

    // init lom api
    $this->lomApiAuth($auth_token);


    $postarr = array(
      'AuthToken' => $auth_token,
      // the comment id
      'comment_id' => $comment_id,
    );

    if (!empty($authorid)) {
      $postarr['authorid'] = $authorid;
    }
    $raw_result = $this->openUrl(
      variable_get(
        'archibald_api_url',
        'https://main-api-dsb.educa.ch'
      ) . '/comment_delete',
      $postarr,
      TRUE
    );

    $result = json_decode($raw_result);

    if ($result == NULL) {
      watchdog(
        'comment delete',
        'comment delete error: no parsable return: !raw_result',
        array('!raw_result' => $raw_result),
        WATCHDOG_ERROR
      );

      return FALSE;
    }

    // session time out
    if ($result->code == 502) {
      $this->lomApiAuth($auth_token, TRUE);
      $postarr['AuthToken'] = $auth_token;
      $result = json_decode($this->openUrl(
          variable_get(
            'archibald_api_url',
            'https://main-api-dsb.educa.ch'
          ) . '/comment_delete',
          $postarr,
          TRUE
        )
      );

      return $result->message;
    }

    if ($result->code != 200) {
      watchdog(
        'comment delete',
        'comment delete error: @code (!message)',
        array(
          '@code' => $result->code,
          '!message' => $result->message,
        ),
        WATCHDOG_ERROR
      );

      return $result->message;
    }
    else {
      return TRUE;
    }
  }
}

