<?php

/**
 * @file
 * class to import Mark21 resources into archibald
 * Mark21 XML 2 Lom - Import / Export
 */
require_once ARCHIBALD_IMPORT_PATH . '/ArchibaldImport.class.php';

/**
 * abstract class to import Mark21 resources into archibald
 * because mark21 is not a well defined standart, is an abstract class
 * It needs to be more specific to handle the ArchibaldMark21 dialect
 */
abstract class ArchibaldMark21 extends ArchibaldImport {

  /**
   * soap client connection
   * @var SoapClient
   */
  protected $soapClient = NULL;
  protected $source;
  public $origSource = array();
  public $taxonsNotFound = array();

  public function __construct() {
    static $soap_client;
    if (empty($soap_client)) {
      $soap_client = $this->soapConnect();
    }
    $this->soapClient = $soap_client;
  }

  /**
   * soap connect to get the values for old ontology server
   *
   * @return SoapClient
   */
  protected function soapConnect() {
    $options = array(
      'soap_version' => SOAP_1_2,
      #'encoding' => 'UTF-8',
      'exceptions' => 0,
    );
    $wsdl = 'http://pbsn3.inf.educa.ch/service/wsdl/ontology';
    try {
      $soap_client = new SoapClient($wsdl, $options);
    }
    catch (SoapFault $fault) {
      echo 'Error: ' . $fault;
      return FALSE;
    }

    return $soap_client;
  }

  /**
   * returns a parsed lom object at given row_id
   *
   * @param integer $row_index
   *   the row id
   * @param boolean $refaktor
   *   refactore after parsing?
   *
   * @return array
   *   the parsed array
   */
  public function getRowAt($row_index, $refaktor = TRUE) {
    if (empty($this->origSource)) {
      $this->prepare();
    }
    if (!isset($this->origSource['collection'])) {
      return FALSE;
    }
    if (!isset($this->origSource['collection']['record'])) {
      return FALSE;
    }
    if (!isset($this->origSource['collection']['record'][$row_index])) {
      return FALSE;
    }
    return $this->parseMarkXml($this->origSource['collection']['record'][$row_index]['datafield'], $refaktor);
  }

  /**
   * generate the table contents for drupal table renderer
   * return array must be "identifier key" => "displayed value like title"
   *
   * @return array
   *   the elements
   */
  public function getTableElements() {
    $c = 0;
    $return_array = array();
    while ($this->source = $this->getRowAt($c++, FALSE)) {
      list($domain, $name, $id) = explode(
        '.',
        $this->getValueAt("035. . .0.a"),
        3
      );

      $title = '';
      $title .= $this->getValueAt("245.0. .0.a");
      $subtitle = $this->getValueAt("245.0. .0.b");
      if (!empty($subtitle)) {
        $title .= " &nbsp; &nbsp; <small>" . $subtitle . "</small>";
      }
      $return_array[$id] = trim($title);
    }
    return $return_array;
  }

  /**
   * get a value from markxml parsed array
   * for example the array is
   * array(
   *  '549' => array(
   *   ' ' => array(
   *      ' ' => array(
   *          array('a' => 'somevalue')
   *      )
   *   )
   *  )
   * )
   *
   * to get the value identifier will be '549. . .a'
   *
   * @param string $value
   *   the ArchibaldMark21 xml key identifer
   * @param string $default
   *
   * @return string
   */
  public function getValueAt($value, $default = "", $row = "") {
    if (is_array($value)) {
      $tmp = $value;
    }
    else {
      $tmp = explode('.', $value);
    }
    if (empty($row)) {
      $tmp_arr = &$this->source;
    }
    else {
      $tmp_arr = &$row;
    }
    foreach ($tmp as $k) {
      if (!isset($tmp_arr[$k])) {
        return $default;
      }

      $tmp_arr = &$tmp_arr[$k];
    }
    return $tmp_arr;
  }

  /**
   * parse and normalize a single Mark21 record
   *
   * @param array $source
   *   the the xml as plain array
   * @param boolean $refaktor
   *   refactor after parsing?
   *
   * @return array
   *   the parsed xml as an array
   */
  public function parseMarkXml($source, $refaktor = TRUE) {

    $return_array = array();
    foreach ($source as $v) {
      $attr = $v['@attributes'];
      if (!isset($return_array[$attr['tag']])) {
        $return_array[$attr['tag']] = array();
      }
      if (!isset($return_array[$attr['tag']][$attr['ind1']])) {
        $return_array[$attr['tag']][$attr['ind1']] = array();
      }
      if (!isset($return_array[$attr['tag']][$attr['ind1']][$attr['ind2']])) {
        $return_array[$attr['tag']][$attr['ind1']][$attr['ind2']] = array();
      }
      if (!isset($v['subfield'])) {
        $v['subfield'] = array();
      }
      // We have multiple entries
      elseif (!isset($v['subfield'][0])) {
        $v['subfield'] = array($v['subfield']);
      }

      $subarray = array();
      foreach ($v['subfield'] as $subfield) {
        $sub_attr = $subfield['@attributes'];
        if (isset($subarray[$sub_attr['code']]) && !is_array($subarray[$sub_attr['code']])) {
          $subarray[$sub_attr['code']] = array($subarray[$sub_attr['code']]);
        }

        if (isset($subarray[$sub_attr['code']]) && is_array($subarray[$sub_attr['code']])) {
          $subarray[$sub_attr['code']][] = $subfield['value'];
          continue;
        }
        if (!isset($subfield['value'])) {
          $subfield['value'] = "";
        }
        $subarray[$sub_attr['code']] = $subfield['value'];
      }
      $return_array[$attr['tag']][$attr['ind1']][$attr['ind2']][] = $subarray;
    }
    ksort($return_array);
    if ($refaktor == TRUE) {
      $this->refactorMarkXML($return_array);
    }

    return $return_array;
  }

  /**
   * Get a ontology term by key
   * @staticvar array
   *   $cache cache variable
   *
   * @param string $term
   *   the term for the ontology server
   *
   * @return string
   *   the ontology string from given term
   */
  protected function getTermByKey(&$term) {
    static $cache;
    if (isset($cache[$term])) {
      $term = $cache[$term];
      return;
    }
    $oterm = $term;
    $term = $this->soapClient->get_individual($term, 'eng');
    if (empty($term)) {
      $this->taxonsNotFound[] = $term = $oterm;
    }
    $cache[$oterm] = $term;
  }

  /**
   * get the new mapping for given oldkey and section
   *
   * @param string $oldkey
   *   the current old key
   * @param string $section
   *   the section like lifecycle or general (optional, default = empty)
   *
   * @return string
   *   the new key
   */
  abstract public function getMapping($oldkey, $section = "");

  /**
   * refactor plan markxml array
   * change ontology keys with their old values and so on.
   * this function will be called after parsing the xml, if needed
   *
   * @param array $return_array
   */
  abstract public function refactorMarkXML(&$return_array);
}

