<?php

/**
 * @file
 * rating publish handler, to submit local ratings into the central catalog
 */

/**
 * Require abstract parent class
 */
module_load_include(
  'php',
  'archibald',
  'includes/dsb_api.class'
);

/**
 * This class publish ratings for ArchibaldLom objects.
 * Precondition for publishing ratings is that ArchibaldLom object is already published.
 *
 * @author cmueller
 */
class ArchibaldRatingsPublish extends ArchibaldAbstractDsbApi {

  /**
   * Definde Prefix for ArchibaldLom-ID
   * This prefix will set from the central catalog to be realy unique
   * @var string
   */
  protected $lom_id_prefix = 'educa###';

  /**
   * Publish all unpublished ratings to Main API Server.
   *
   * @return boolean status
   */
  public function cron() {
    // Check if cron was stopped due to uploading problems
    if (variable_get('archibald_upload_error', 0) == 1) {
      if (function_exists('drush_log')) {
        drush_log(
          'Ratings Publish: Upload error bit setted', 'error'
        );
      }
      return FALSE;
    }

    $maintenance_max_execution_time = ini_get('max_execution_time');
    if ($maintenance_max_execution_time == 0) {
      $maintenance_max_execution_time = 240;
    }

    if ($maintenance_max_execution_time > 20) {
      $maintenance_max_execution_time -= 10;
    }

    if (!lock_acquire('ratings_publish', ($maintenance_max_execution_time + 1))) {

      if (function_exists('drush_log')) {
        drush_log(
          'Ratings Publish: action is locked, perhaps a gui publish process is open', 'error'
        );
      }
      return FALSE;
    }

    $processed_ratings = 0;
    $maintenance_start_time = time();

    // Make sure that only ratings are published which belong
    // to published ArchibaldLom objects
    $res = db_query("
      SELECT DISTINCT ls.lom_id, r.rid, r.uid, r.rating
      FROM {archibald_lom_stats} AS ls
      INNER JOIN {archibald_lom} AS l ON (ls.lom_id=l.lom_id)
      LEFT JOIN {archibald_ratings} AS r ON (r.lom_id=l.lom_id)
      WHERE l.deleted IS NULL
      AND ls.publication_progress=100
      AND r.ident_dsb='' ");

    while ($rating = $res->fetchObject()) {
      // processingtime exceeded
      if (($maintenance_start_time + ($maintenance_max_execution_time / 2)) <= time()) {
        break;
      }

      // on upload attempt we run into a fatal permanent error
      if (variable_get('archibald_upload_error') == 1) {
        break;
      }

      // init content partner settings
      $this->loadContentPartnerByLomId($lom_id);

      if (!$this->contentPartnerCanCentral()) {
        // this content partner is not allowed to access central catalog
        continue;
      }

      $result = $this->ratingAdd(
        $rating->lom_id, $rating->rating, $rating->uid
      );

      $ident_dsb = NULL;
      if ($result != FALSE) {
        if (function_exists('drush_log')) {
          drush_log(
            'Ratings Publish: sucessfull published rating with id ' .
            $rating->rid, 'ok'
          );
        }
        $ident_dsb = $result;
      }
      else {
        $ident_dsb = 'error';
      }
      db_query(
        'UPDATE {archibald_ratings} SET ident_dsb=:ident_dsb WHERE rid=:rid',
        array(
          ':ident_dsb' => $ident_dsb,
          ':rid' => $rating->rid,
        )
      );

      $processed_ratings++;
    }

    if ($processed_ratings == 0) {
      if (function_exists('drush_log')) {
        drush_log('Ratings Publish: nothing to do', 'ok');
      }
    }

    lock_release('ratings_publish');
    return TRUE;
  }

  /**
   * Call the remote function 'ratingAdd' on Main API Server
   *
   * @param string $lom_id
   *   ArchibaldLom ID which rating belongs to
   * @param int $rating
   *   rating value (1-5)
   *   if rating is smaller that 1 set to 1
   *   if rating is greater than 5 set to 5
   *   Multiply with 20 to get value in percent
   * @param string $authorid
   *   ID of user which submitted th rating
   *   if ID is 0 (anonymous users) generate random ID
   * @param boolean $verbose
   *   default:TRUE
   *   set to FALSE to only return error messages and dont call execptions
   *
   * @return mixed
   *  status
   *  if fail return FALSE
   *  otherwise return the central dsb rating id
   */
  public function ratingAdd($lom_id, $rating, $authorid = '', $verbose = TRUE) {
    // init content partner settings
    $this->loadContentPartnerByLomId($lom_id);

    if (!$this->contentPartnerCanCentral()) {
      // this content partner is not allowed to access central catalog
      return FALSE;
    }

    // init lom api
    $this->lomApiAuth($auth_token);

    // Check that rating is between 1 and 5 and transform it to percent
    $rating = round($rating);
    if ($rating < 1 && $rating <= 5) {
      $rating = 1;
    }

    if ($rating > 5) {
      $rating = 5;
    }
    $rating = $rating * 20;

    // Check usser id: If 0 use random string because multiple anonymous users
    // are allowed to rate a ArchibaldLom object
    if ($authorid == 0) {
      $authorid = $this->createRandomString();
    }

    // Prepare POST Variables
    $postarr = array(
      'AuthToken' => $auth_token,
      'lom_id' => $this->lom_id_prefix . $lom_id,
      'authorid' => $authorid,
      'percent' => $rating,
    );

    $raw_result = $this->openUrl(variable_get('archibald_api_url', 'https://main-api-dsb.educa.ch') . '/rating_add', $postarr, TRUE);

    $result = json_decode($raw_result);
    if (empty($result)) {
      if (function_exists('drush_log')) {
        drush_log(
          'Rating Publish: no parsable return: ' . $raw_result, 'error'
        );

        watchdog(
          'rating add', 'rating add error: cannot parse result: !raw_result',
          array(
            '!raw_result' => $raw_result,
          ), WATCHDOG_ERROR, 'archibald/' . $lom_id
        );
      }
      return FALSE;
    }

    // session time out
    if ($result->code == 502) {
      $this->lomApiAuth($auth_token, TRUE);
      $postarr['AuthToken'] = $auth_token;
      $result = json_decode(
        $this->openUrl(
          variable_get(
            'archibald_api_url', 'https://main-api-dsb.educa.ch'
          ) .
          '/rating_add', $postarr, TRUE
        )
      );
    }

    if ($result->code != 200) {
      if (function_exists('drush_log') && $verbose == TRUE) {
        drush_log('rating add error: ' . $result->code . ' (' . $result->message . ')', 'error');

        watchdog(
          'rating add', 'rating add error: @code (!message)', array(
            '@code' => $result->code,
            '!message' => $result->message,
          ), WATCHDOG_ERROR, 'archibald/' . $lom_id
        );
      }
      return FALSE;
    }
    else {
      return $result->data->rating_id;
    }

    return FALSE;
  }
}

