<?php

/**
 * @file
 * class for indexing lom resources into the solr index
 */

/**
 * require ArchibaldLomContributer
 * for rendering vCards to plain data
 *
 * this class could used without drupal
 * only replace ArchibaldLomSolrDrupal with abstraction for your system
 *
 */
require_once dirname(__FILE__) . '/LomContributer.class.php';

/**
 * require ArchibaldLomPublish
 * for loading lom object from database
 */
require_once dirname(__FILE__) . '/LomPublish.class.php';

/**
 * require curriculum_educa
 * for getting default curriculum values
 */
require_once dirname(__FILE__) . '/curriculum/curriculum_educa.class.php';

/**
 * require curriculum_per
 * for getting per curriculum values
 */
require_once dirname(__FILE__) . '/curriculum/curriculum_per.class.php';

/**
 * lom to solr document converter
 *
 * @author Heiko Henning <h.henning@educa.ch>
 */
class ArchibaldLomSolr extends ArchibaldLomSolrDrupal {

  /**
   * a list of processed lom objects
   *  array( lom_id=>version,
   *         lom_id=>version,
   *         .....
   *       )
   * @var array
   */
  protected $processed = array();

  /**
   * a list apache solr documents
   * @var array
   */
  protected $solrDocs = array();

  /**
   * start generating solr objects form lom object
   *
   * @param string $lom_id
   * @param boolean $search_for_parents
   *   default TRUE
   *   we dont like endless loops
   *   lookup only 1 recurive level
   * @param array $documents_to_delete
   *   called by refference
   *   list lom_id`s with are to delte from index
   *
   * @return boolean status
   */
  public function index($lom_id, $search_for_parents = TRUE, &$documents_to_delete = array()) {
    $version_to_index = ArchibaldLomSaveHandler::getLocalPublished($lom_id);

    // if it is not published, dont index it
    if (!empty($version_to_index)) {
      $lom = $this->loadLom($lom_id, $version_to_index);
      if (!($lom instanceof ArchibaldLom)) {
        return FALSE;
      }

      if ($lom->getLifeCycle()->getStatus()->getValue() == 'deleted') {
        $status_valid_for_indexing = FALSE;
      }
      else {
        $status_valid_for_indexing = TRUE;
      }
    }

    //Resource is not local published so dont index it
    else {
      $status_valid_for_indexing = FALSE;
      $lom = $this->loadLom($lom_id);
      if (!($lom instanceof ArchibaldLom)) {
        return FALSE;
      }
    }

    if (!$status_valid_for_indexing) {
      $status_val = $lom->getLifeCycle()->getStatus()->getValue();
      $this->log('Delete @lom_id for @status',  array('@lom_id' => $lom_id, '@status' => $status_val), 'ok');

      $this->processed[$lom->getLomId()] = $lom->getVersion();

      // look up for childs and index them as main items
      foreach ($lom->getRelation() as $relation) {
        // found a parent
        if ($relation->getKind()->getValue() == ArchibaldLomDataRelation::KIND_HAS_PART) {

          $resource = $relation->getResource();
          $child_lom_id = $resource->getIdentifier(0)->getEntry();

          if (!empty($child_lom_id)) {
            $this->index($child_lom_id, FALSE, $documents_to_delete);
          }
        }
      }

      return 'delete';
    }

    // look up for parents and start indexing there to index every time
    // the whole collection
    if ($search_for_parents === TRUE) {
      foreach ($lom->getRelation() as $relation) {
        // found a parent
        if ($relation->getKind()->getValue() == ArchibaldLomDataRelation::KIND_IS_PART_OF) {
          $resource = $relation->getResource();
          $parent_lom_id = $resource->getIdentifier(0)->getEntry();
          if (!empty($parent_lom_id)) {
            $this->log(
              'Using collection parent @parent_lom_id for @lom_id',
              array(
                '@lom_id' => $lom_id,
                '@parent_lom_id' => $parent_lom_id,
              ),
              'notice'
            );

            return $this->index($parent_lom_id, FALSE);
          }
        }
      }
    }

    // determine all possible languages for this object
    $langs = array_intersect(
      array_keys($lom->getGeneral()->getTitle()->getStrings()),
      array_keys($lom->getGeneral()->getDescription()->getStrings())
    );

    if (empty($langs)) {
      $this->log('No processable languages found for @lom_id', array('@lom_id' => $lom_id), 'error');
    }

    foreach ($langs as $lang) {
      $this->log('Process @lom_id for language: @lang', array('@lom_id' => $lom_id, '@lang' => $lang), 'ok');

      // determine all childs
      $childs = array();
      foreach ($lom->getRelation() as $relation) {
        // found a parent
        if ($relation->getKind()->getValue() == ArchibaldLomDataRelation::KIND_HAS_PART) {

          $resource = $relation->getResource();
          $child_lom_id = $resource->getIdentifier(0)->getEntry();
          if (!empty($child_lom_id)) {
            $childs[] = $child_lom_id;
          }
        }
      }

      $document = $this->indexCollectionHead($lom, $lang, (empty($childs)) ? 0 : 2);

      $this->processed[$lom->getLomId()] = $lom->getVersion();

      // we need to add the main options all also to the child fields,
      // otherwise facetting will not work
      $this->indexCollectionChild($document, $document);


      foreach ($childs as $child_lom_id) {
        $child_lom = $this->loadLom($child_lom_id);

        $child_document = $this->indexCollectionHead($child_lom, $lang, 1);
        $this->indexCollectionChild($document, $child_document);
        $this->postprocess($child_document);

        $this->solrDocs[] = $child_document;
        $this->processed[$child_lom->getLomId()] = $child_lom->getVersion();

        // delete standalone child
        $documents_to_delete[] = $child_lom->getLomId();

        $this->log('Process child @lom_id for @lang language', array('@lom_id' => $child_lom_id, '@lang' => $lang), 'ok');
      }

      $this->postprocess($document);
      $this->solrDocs[] = $document;
    }

    return 'insert';
  }

  /**
   * create a new solr object and index it
   *
   * @param ArchibaldLom $lom
   * @param string $lang
   * @param integer $collection_type
   *
   * ***** Solr Object Structur *****
   * attribute name prefixes:
   *
   * ss = string single
   * sm = string multi

   * ts = text singe
   * tm = text multi
   *
   * hs = integer single
   * hm = integer multi
   *
   * hts = tiny integer single
   * htm = tiny integer multi
   *
   * For further informations
   * @see sites/all/moduels/apachesolr/schema.xml
   *
   * *******************************
   *
   * entity_type = lom
   * ss_entity_id  = lom_id
   * label = general.title
   * path = archibald/$lom_id
   * content = general.description
   * teaser = substr(general.description, 0, 300)...
   *
   * tm_keywords
   * tm_coverage
   * tags_keywords = implode(' ', tm_keywords);
   * tags_coverage = implode(' ', tm_coverage);
   *
   * ss_version
   * ss_location
   *
   * twm_languages
   *
   * hs_preview_image
   *
   * ts_other_plattform_requirements
   * ts_education_description
   *
   * tm_contributer
   *
   * sm_learning_resourcetype
   * sm_intendet_enduserrole
   *
   * htm_typical_age_range
   *      here we store each single year with is in selcted range
   *      infinity is 99
   * ss_typical_age_range    here we store the value like in lom object
   *
   * ss_difficulty here the search need to search for multi values
   *
   * hs_typical_learningtime  value in minutes
   *
   * hts_cost  1/0
   * htc_copyright_and_other_restrictions  1/0
   *
   * sm_context
   *
   * sm_education_level
   * sm_discipline
   *      Both from default curriculum and reverse selected
   *
   * Dont store: rights_description
   *
   * hts_collection
   *      0 = not in collection
   *      1 = in a collection
   *      2 = in a collection and collection head
   *
   */
  protected function indexCollectionHead($lom, $lang, $collection_type) {
    $document = new ApacheSolrDocument();
    $document->id = apachesolr_document_id(
      $lom->getLomId() . '/' . $lang, 'lom'
    );

    $document->site = url(NULL, array('absolute' => TRUE));
    $document->hash = apachesolr_site_hash();
    $document->entity_type = 'lom';
    // here we only can store numeric long values
    $document->entity_id = 0;
    $document->ss_entity_id = $lom->getLomId();

    $document->hts_collection = $collection_type;

    // process title
    $lom_title = $lom->getGeneral()->getTitle()->getString($lang);
    $document->label = $this->cleanText($lom_title);

    $path           = 'archibald/' . $lom->getLomId();
    $document->url  = url($path, array('absolute' => TRUE));
    $document->path = $path;

    // process description
    $lom_text          = $lom->getGeneral()->getDescription()->getString($lang);
    $document->content = $this->cleanText($lom_text);
    $document->teaser  = truncate_utf8(strtr($document->content, array("\n" => ' ', "\r" => '')), 300, TRUE);

    $document->ss_language = $lang;

    $this->processKeywords($lom, $document, $lang);

    // process version and location
    $value = ArchibaldLom::getLangstringText($lom->getLifeCycle()->getVersion());
    if (!empty($value)) {
      $document->setField('ss_version', $value);
    }

    $value = $lom->getTechnical()->getLocation(0);
    if (!empty($value)) {
      $document->setField('ss_location', $value);
    }

    // process languages
    foreach ($lom->getGeneral()->getLanguage() as $general_language) {
      $document->setMultiValue('twm_languages', $general_language);
    }

    // process other Plattform Requirements
    $lang_string = $lom->getTechnical()->getOtherPlattformRequirements();
    if ($lang_string instanceof ArchibaldLomDataLangString) {
      $text = $this->cleanText($lang_string->getString($lang));
      if (!empty($text)) {
        $document->setField('ts_other_plattform_requirements', $text);
      }
    }

    // process educational description
    $lang_string = $lom->getEducation()->getDescription();
    if ($lang_string instanceof ArchibaldLomDataLangString) {
      $text = $this->cleanText($lang_string->getString($lang));
      if (!empty($text)) {
        $document->setField('ts_education_description', $text);
      }
    }

    // process contributer
    foreach ($lom->getLifeCycle()->getContribute() as $contributer_id => $contribute) {
      $tm_contributer = '';
      /*
       * $role = archibald_get_term_by_key($contribute->getRole(), 'lc_cont_role', TRUE);
       * $tm_contributer  .= $role[$lang].' ';
       */


      $contributer = new ArchibaldLomContributer($contribute->entity[0]);
      $contributer_data = $contributer->getData();
      foreach ($contributer_data as $entety) {
        $tm_contributer .= $entety . ' ';
      }
      $author_name = '';
      if (!empty($contributer_data['firstname'])) {
        $author_name .= ' ' . $contributer_data['firstname'];
      }
      if (!empty($contributer_data['lastname'])) {
        $author_name .= ' ' . $contributer_data['lastname'];
      }
      if (!empty($contributer_data['organisation'])) {
        $author_name .= ' ' . $contributer_data['organisation'];
      }
      $author_name = trim($author_name);
      if (!empty($author_name)) {
        $document->setMultiValue('sm_author', $this->cleanText($author_name));
      }

      $document->setMultiValue('tm_contributer', $this->cleanText($tm_contributer));
    }

    // process lerning resource type
    $this->processVocabular(
      $document, 'sm_learning_resourcetype', $lom->getEducation()->getLearningResourceType(), 'learning_resourcetype', $lang
    );

    // process intendet end user role
    $this->processVocabular(
      $document, 'sm_intendet_enduserrole', $lom->getEducation()->getIntendedEndUserRole(), 'intendet_enduserrole', $lang
    );

    // process preview image
    $default_value = $lom->getTechnical()->getPreviewImage();
    preg_match('/archibald_file\/([0-9]+)\//', $default_value, $old_fid);
    $document->setField('hs_preview_image', (isset($old_fid[1])) ? $old_fid[1] : 0);

    // process typical age range
    $lang_string = $lom->getEducation()->getTypicalAgeRange(0);
    if ($lang_string instanceof ArchibaldLomDataLangString) {
      $document->setField('ss_typical_age_range', $lang_string->getString('en'));

      if (preg_match('/^([\-U0-9]{0,3})\-([\-U0-9]{0,3})$/i', (string) $lang_string->getString('en'), $hits)) {
        $default_value_from = min((INT) $hits[1], (INT) $hits[2]);
        $default_value_to = max((INT) $hits[1], (INT) $hits[2]);
      }
      if (empty($default_value_from)) {
        $default_value_from = 0;
      }
      if (empty($default_value_to)) {
        $default_value_to = 99;
      }
      for ($j = $default_value_from; $j <= $default_value_to; $j++) {
        $document->setMultiValue('htm_typical_age_range', $j);
      }
    }

    // process educational difficulty
    $vocabulary = $lom->getEducation()->getDifficult();
    if ($vocabulary instanceof ArchibaldLomDataVocabulary) {
      $document->setField('ss_difficulty', $vocabulary->getValue());
      $human_value = archibald_get_term_by_key($vocabulary->getValue(), 'difficulty', FALSE, $lang);

      if (!empty($human_value)) {
        $document->setMultiValue('tm_vocabular_text', $human_value);
      }
    }

    // process typical lerning time
    $typical_learning_time = $lom->getEducation()->getTypicalLearningTime();
    if ($typical_learning_time instanceof ArchibaldLomDataDuration) {
      $document->setField('hs_typical_learningtime', $typical_learning_time->getDurationIn('minutes'));
    }

    // process rights cost
    $vocabulary = $lom->getRights()->getCost();
    if ($vocabulary instanceof ArchibaldLomDataVocabulary) {
      $document->setField('hts_cost', ($vocabulary->getValue() == 'yes') ? 1 : 0);
    }

    // process rights copyrigth and other restrictions
    $vocabulary = $lom->getRights()->getCopyrightAndOtherRestrictions();
    if ($vocabulary instanceof ArchibaldLomDataVocabulary) {
      $document->setField('htc_copyright_and_other_restrictions', ($vocabulary->getValue() == 'yes') ? 1 : 0);
    }

    // process context
    $document->sm_context = array();
    $this->processVocabular($document, 'sm_context', $lom->getEducation()->getContext(), 'education_context', $lang);

    // process education level and dicipline
    $curr_handler = new ArchibaldCurriculumEduca();
    $curr_handler->setLomClassification($lom->getClassification());
    foreach ($curr_handler->getEntryObjects() as $entry) {
      $vocabular_text = '';
      foreach ($entry->educationalLevelIds as $educational_level) {
        $document->setMultiValue('sm_education_level', $educational_level);

        $human_value = archibald_get_term_by_key($educational_level, 'educa_school_levels', FALSE, $lang);

        if (!empty($human_value)) {
          $vocabular_text .= ' ' . $human_value;
        }
      }
      if (!empty($vocabular_text)) {
        $document->setMultiValue('tm_vocabular_text', trim($vocabular_text));
      }

      $vocabular_text = '';
      foreach ($entry->disciplineIds as $discipline) {
        $document->setMultiValue('sm_discipline', $discipline);

        $human_value = archibald_get_term_by_key($discipline, 'educa_school_subjects', FALSE, $lang);

        if (!empty($human_value)) {
          $vocabular_text .= ' ' . $human_value;
        }
      }
      if (!empty($vocabular_text)) {
        $document->setMultiValue('tm_vocabular_text', trim($vocabular_text));
      }
    }


    // process per curriculum
    $curr_handler = new ArchibaldCurriculumPer();
    $curr_handler->setLomCurriculum($lom->getCurriculum());
    $curr_handler->solrIndex($document);


    // get company logo if it is not default
    $variables['lom'] = &$lom;
    archibald_get_lom_for_template_company_logo($variables);
    if (!empty($variables['company_logo'])) {
      $document->setMultiValue('hs_company_logo', (INT) $variables['company_logo']);
    }

    return $document;
  }

  /**
   * integrate data from child solr object into head solr object
   *
   * @param ApacheSolrDocument $document
   * @param ApacheSolrDocument $child_document
   *
   * ***** Solr Object Structur *****
   * attribute name prefixes: see Doctype of indexCollectionHead()
   *
   * tm_child_label = label
   * tm_child_content = content
   *
   * sm_child_entity_id = ss_entity_id
   *
   * tm_child_keywords = tm_keywords
   * tm_child_coverage = tm_coverage
   *
   * twm_child_languages = twm_languages
   *
   * tm_child_other_plattform_requirements = ts_other_plattform_requirements
   * tm_child_education_description = ts_education_description
   *
   * tm_child_contributer = tm_contributer
   *
   * sm_child_learning_resourcetype = sm_learning_resourcetype
   * sm_child_intendet_enduserrole = sm_intendet_enduserrole
   *
   * htm_child_typical_age_range = htm_typical_age_range
   *      here we store each single year with is in selcted range
   *      infinity is 99
   *
   * sm_child_difficulty = ss_difficulty
   *
   * hm_child_typical_learningtime = hs_typical_learningtime
   * value in minutes
   *
   * sm_child_context = sm_context
   *
   * sm_child_education_level = sm_education_level
   * sm_child_discipline = sm_discipline
   *      Both from default curriculum and reverse selected
   *
   * ***********
   *
   * dont scare about double existing values,
   * this will be cleaned up by: postprocess() function
   */
  protected function indexCollectionChild(&$document, $child_document) {
    $document->setMultiValue('tm_child_label', $child_document->label);

    $document->setMultiValue('tm_child_content', $child_document->content);

    // the child lom id is the only object we dont like
    // to have content from the master object in
    if ($document->ss_entity_id != $child_document->ss_entity_id) {
      $document->setMultiValue('sm_child_entity_id', $child_document->ss_entity_id);
    }

    if (!empty($child_document->tm_keywords)) {
      foreach ($child_document->tm_keywords as $value) {
        $document->setMultiValue('tm_child_keywords', $value);
      }
    }

    if (!empty($child_document->tm_coverage)) {
      foreach ($child_document->tm_coverage as $value) {
        $document->setMultiValue('tm_child_coverage', $value);
      }
    }

    if (!empty($child_document->twm_languages)) {
      foreach ($child_document->twm_languages as $value) {
        $document->setMultiValue('twm_child_languages', $value);
      }
    }

    if (!empty($child_document->ts_other_plattform_requirements)) {
      $document->setMultiValue('tm_child_other_plattform_requirements', $child_document->ts_other_plattform_requirements);
    }

    if (!empty($child_document->ts_education_description)) {
      $document->setMultiValue('tm_child_education_description', $child_document->ts_education_description);
    }

    if (!empty($child_document->tm_contributer)) {
      foreach ($child_document->tm_contributer as $value) {
        $document->setMultiValue('tm_child_contributer', $value);
      }
    }

    if (!empty($child_document->sm_learning_resourcetype)) {
      foreach ($child_document->sm_learning_resourcetype as $value) {
        $document->setMultiValue('sm_child_learning_resourcetype', $value);
      }
    }

    if (!empty($child_document->sm_intendet_enduserrole)) {
      foreach ($child_document->sm_intendet_enduserrole as $value) {
        $document->setMultiValue('sm_child_intendet_enduserrole', $value);
      }
    }

    if (!empty($child_document->htm_typical_age_range)) {
      foreach ($child_document->htm_typical_age_range as $value) {
        $document->setMultiValue('htm_child_typical_age_range', $value);
      }
    }

    if (!empty($child_document->ss_difficulty)) {
      $document->setMultiValue('sm_child_difficulty', $child_document->ss_difficulty);
    }

    if (!empty($child_document->hs_typical_learningtime)) {
      $document->setMultiValue('hm_child_typical_learningtime', $child_document->hs_typical_learningtime);
    }

    if (!empty($child_document->sm_context)) {
      foreach ($child_document->sm_context as $value) {
        $document->setMultiValue('sm_child_context', $value);
      }
    }

    if (!empty($child_document->sm_education_level)) {
      foreach ($child_document->sm_education_level as $value) {
        $document->setMultiValue('sm_child_education_level', $value);
      }
    }

    if (!empty($child_document->sm_discipline)) {
      foreach ($child_document->sm_discipline as $value) {
        $document->setMultiValue('sm_child_discipline', $value);
      }
    }

    if (!empty($child_document->tm_vocabular_text)) {
      foreach ($child_document->tm_vocabular_text as $value) {
        $document->setMultiValue('tm_child_vocabular_text', $value);
      }
    }

    if (!empty($child_document->sm_author)) {
      foreach ($child_document->sm_author as $value) {
        $document->setMultiValue('sm_child_author', $value);
      }
    }
  }

  /**
   * return a list of processed lom objects
   *  array( lom_id=>version,
   *         lom_id=>version,
   *         .....
   *       )
   *
   * @return array
   */
  public function getProcessed() {
    return $this->processed;
  }

  /**
   * return a list solr index documents
   *
   * @return array
   */
  public function getDocs() {
    return $this->solrDocs;
  }

  /**
   * extract keywords from lom object and add it to apache solr object
   *
   * @param ArchibaldLom $lom
   * @param ApacheSolrDocument $document
   * @param string $lang
   */
  protected function processKeywords(ArchibaldLom &$lom, ApacheSolrDocument &$document, $lang) {
    $lom_general = $lom->getGeneral();
    foreach ($lom_general->getKeyword() as $keyword) {
      if ($keyword instanceof ArchibaldLomDataLangString) {
        $string = $keyword->getString($lang);
        if (!empty($string)) {
          $document->setMultiValue('tm_keywords', $string);
        }
      }
    }

    foreach ($lom_general->getCoverage() as $keyword) {
      if ($keyword instanceof ArchibaldLomDataLangString) {
        $string = $keyword->getString($lang);
        if (!empty($string)) {
          $document->setMultiValue('tm_coverage', $string);
        }
      }
    }
  }

  /**
   * post process solr document
   * fill up tags_* variables
   *
   * @param ApacheSolrDocument $document
   */
  protected function postprocess(ApacheSolrDocument &$document) {
    $keywords = $document->getField('tm_keywords');
    if ($keywords != FALSE) {
      $document->setField('tags_keywords', implode(' ', $keywords['value']));
    }

    $keywords = $document->getField('tm_coverage');
    if ($keywords != FALSE) {
      $document->setField('tags_coverage', implode(' ', $keywords['value']));
    }

    $multi_fields = array('sm_label',
      'tm_content',
      'tm_child_keywords',
      'tm_child_coverage',
      'tm_keywords',
      'tm_coverage',
      'twm_child_languages',
      'tm_child_other_plattform_requirements',
      'tm_child_education_description',
      'tm_contributer',
      'tm_child_contributer',
      'sm_learning_resourcetype',
      'sm_child_learning_resourcetype',
      'sm_intendet_enduserrole',
      'sm_child_intendet_enduserrole',
      'htm_typical_age_range',
      'htm_child_typical_age_range',
      'sm_child_difficulty',
      'hm_child_typical_learningtime',
      'sm_context',
      'sm_child_context',
      'sm_education_level',
      'sm_child_education_level',
      'sm_discipline',
      'sm_child_discipline',
      'tm_vocabular_text',
      'tm_child_vocabular_text',
      'sm_author',
      'sm_child_author',
    );

    // uniqui field values
    foreach ($multi_fields as $multi_field) {
      $data = $document->getField($multi_field);
      if (!empty($data)) {
        $values = array();
        foreach ($data['value'] as $val) {
          $key = md5($val);
          if (isset($values[$key])) {
            $values[$key]['ammount']++;
          }
          else {
            $values[$key] = array('value' => $val, 'ammount' => 1);
          }
        }
        unset($document->$multi_field);
        foreach ($values as $value) {
          // we dont set boost here, cause:
          // An Index-time boost on a value of a multiValued field applies
          // to all values for that field.
          // see http://wiki.apache.org/solr/SolrRelevancyFAQ#index-time_boosts
          $document->setMultiValue($multi_field, $value['value']);
        }
      }
    }
  }

  /**
   * strip all html from text an avoid word gluing, also do br2nl
   *
   * @param string $text
   *
   * @return string
   */
  public function cleanText($text) {
    $text = strtr($text, array("\r" => ''));
    $text = strtr($text, array("<br />\n" => "\n", "<br\>\n" => "\n", "<br \>\n" => "\n"));
    // br2nl
    $text = preg_replace('/\<br(\s*)?\/?\>/i', "\n", $text);

    // add space for and after each html tag to avoid glued words
    $text = str_replace(array('<', '>'), array(' <', '> '), $text);
    $text = strip_tags($text);

    return trim($text);
  }

  /**
   * process a vocabular,
   * save it to lom document and store it also to  vocabular_text
   *
   * @param ApacheSolrDocument $document
   * @param sring $variable
   *   variable of solr docuemtn to store in
   * @param array $vocabular
   *   list of ArchibaldLomDataVocabulary
   * @param string $vocabular_name
   *   name of vocabular fomr ontology::read()
   *   see: ArchibaldLomDataVocabulary::get_human_string()
   * @param string $lang
   */
  protected function processVocabular(ApacheSolrDocument &$document, $variable, $vocabular, $vocabular_name, $lang) {

    $vocabular_text = '';
    foreach ($vocabular as $vocab_entry) {
      $value = $vocab_entry->getValue();
      $human_value = archibald_get_term_by_key($value, $vocabular_name, FALSE, $lang);

      if (!empty($human_value)) {
        // with this if, cause we only like to add option
        // the we realy have in our ontology
        $document->setMultiValue($variable, $value);

        $vocabular_text .= ' ' . $human_value;
      }
    }
    if (!empty($vocabular_text)) {
      $document->setMultiValue('tm_vocabular_text', trim($vocabular_text));
    }
  }
}

abstract class ArchibaldLomSolrDrupal {

  /**
   * load a lom object by id
   *
   * @param string $lom_id
   * @param string $version_id
   *
   * @return ArchibaldLom
   */
  protected function loadLom($lom_id, $version_id = '') {
    $lom = archibald_load($lom_id, $version_id);
    if (!($lom instanceof ArchibaldLom)) {
      return FALSE;
    }

    $lom = ArchibaldLomPublish::completeLomObject($lom);
    $version_id = $lom->getVersion();

    // if it was deleted set lifeCycle status as deleted
    $deleted = db_query(
      'SELECT deleted
        FROM {archibald_lom}
        WHERE lom_id=:lom_id
        AND version=:version
        LIMIT 1',
      array(
        'lom_id' => $lom_id,
        ':version' => $version_id,
      )
    )
    ->fetchColumn(0);

    if (!empty($deleted)) {
      $lom_life_cycle = $lom->getLifeCycle();
      $lom_life_cycle->setStatus(new ArchibaldLomDataVocabulary('deleted'));
      $lom->setLifeCycle($lom_life_cycle);
    }

    return $lom;
  }

  /**
   * log message processor
   *
   * @param string $messages
   *   english message string with vars within
   * @param array $variables
   * @param string $level
   *  ok / notice / error
   */
  protected function log($messages, $variables, $level) {
    if (function_exists('drush_log')) {
      drush_log(t($messages, $variables), $level);
    }

    $watchdog_levels = array(
      'ok' => WATCHDOG_NOTICE,
      'notice' => WATCHDOG_DEBUG,
      'error' => WATCHDOG_ERROR,
    );
    watchdog('solrIndex', $messages, $variables, $watchdog_levels[$level]);
  }
}

