<?php
/**
 * @file
 * this file contains a LOM metadata field names
 */


/**
 * This function returns an array with all LOM metadata-field names (machine-readable) and labels
 *
 * The LOM metadata fields have following structure:
 * To form a field name use points to separate the levels, e.g. "education.learningResourceType"
 *
 * @return array
 *   A list of all lom fields including titles and requrements.
 *
 * general
 *    description
 *    language
 *    identifier
 *        entry
 *        catalog
 *    keyword
 *    coverage
 *
 * technical
 *    location
 *    otherPlattformRequirements
 *
 * education
 *    description
 *    intendedEndUserRole
 *    typicalLearningTime
 *    difficulty
 *    learningResourceType
 *    typicalAgeRange
 *
 * lifecycle
 *    version
 *    contributor
 *        role
 *        entity
 *
 * right
 *     description
 *     cost
 *     copyright
 *     licence
 *
 * classification
 *     context
 *     curriculums
 *
 * relation
 *     resource
 *     relation
 */
function archibald_get_field_info() {

  $general_fields = array(
    'title' => array(
      'label' => t('Title'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_REQUIRED,
    ),
    'description' => array(
      'label' => t('General description'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_REQUIRED,
    ),
    'language' => array(
      'label' => t('Language'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),
    'identifier' => array(
      'label' => t('Identifier'),
      'subfields' => array(
        'entry' => array(
          'label' => t('Entry'),
        ),
        'catalog' => array(
          'label' => t('Catalogue'),
        ),
        'title' => array(
          'label' => t('Title'),
        ),
      ),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_REQUIRED,
    ),
    'keyword' => array(
      'label' => t('Tags'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),
    'coverage' => array(
      'label' => t('Coverage'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
  );

  $technical_fields = array(
    'location' => array(
      'label' => t('Location'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
    'otherPlattformRequirements' => array(
      'label' => t('Technical requirements'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),
    'previewImage' => array(
      'label' => t('Preview image'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),
  );

  $lifecycle_fields = array(
    'version' => array(
      'label' => t('Version'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
    'contributor' => array(
      'label' => t('Contributor'),
      'subfields' => array(
        'role' => array(
          'label' => t('Role'),
        ),
        'entity' => array(
          'label' => t('Entity'),
        ),
      ),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
  );

  $right_fields = array(
    'description' => array(
      'label' => t('Licence'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
    'cost' => array(
      'label' => t('Cost'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
    'copyright' => array(
      'label' => t('Copyright and other restrictions'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
  );

  $education_fields = array(
    'intendedEndUserRole' => array(
      'label' => t('Intended end user role'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),
    'typicalLearningTime' => array(
      'label' => t('Typical learning time'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
    'difficulty' => array(
      'label' => t('Difficulty'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
    'description' => array(
      'label' => t('Educational description'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),
    'learningResourceType' => array(
      'label' => t('Learning resource type'),
      'minimum_requirements' => (archibald_can_publish_central() ? ARCHIBALD_RES_FIELD_REQUIRED : ARCHIBALD_RES_FIELD_RECOMMEND),
    ),
    'typicalAgeRange' => array(
      'label' => t('Typical age range'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),
    'aggregationLevel' => array(
      'label' => t('Aggregation level'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),

  );

  $classification_fields = array(
    'context' => array(
      'label' => t('Context'),
      'minimum_requirements' => (archibald_can_publish_central() ? ARCHIBALD_RES_FIELD_REQUIRED : ARCHIBALD_RES_FIELD_RECOMMEND),
    ),
    'curriculums' => array(
      'label' => t('Curricula'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_RECOMMEND,
    ),
  );


  $relation_fields = array(
    'resource' => array(
      'label' => t('Description'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
    'relation' => array(
      'label' => t('Relation'),
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
  );

  $lom_sections = array(
    'general' => array(
      'label' => t('General information'),
      'fields' => $general_fields,
    ),
    'technical' => array(
      'label' => t('Technical requirements'),
      'fields' => $technical_fields,
    ),
    'lifecycle' => array(
      'label' => t('Lifecycle information'),
      'fields' => $lifecycle_fields,
    ),
    'education' => array(
      'label' => t('Educational information'),
      'fields' => $education_fields,
    ),
    'classification' => array(
      'label' => t('Curricula'),
      'fields' => $classification_fields,
    ),
    'relation' => array(
      'label' => t('Relations'),
      'fields' => $relation_fields,
      'minimum_requirements' => ARCHIBALD_RES_FIELD_OPTIONAL,
    ),
    'right' => array(
      'label' => t('Rights'),
      'fields' => $right_fields,
    ),
  );

  return $lom_sections;
}

/**
 * Get a list of minimum requirements for all lom fields.
 *
 * @param string $field_name
 *   Id of field to get minimum requirements from.
 *   If empty the function will return the full array.
 *
 * @return array
 *   A list of minumum requirements for all lom fields.
 *   The key is concatinated from section_id dot field_id
 *   The value is  ARCHIBALD_RES_FIELD_OPTIONAL, ARCHIBALD_RES_FIELD_RECOMMEND or ARCHIBALD_RES_FIELD_REQUIRED
 */
function archibald_get_field_minimum_requirements($field_name = NULL) {
  static $minimum_requirements = NULL;

  if (is_null($minimum_requirements)) {
    $minimum_requirements = array();
    foreach (archibald_get_field_info() as $section_id => $section) {
      foreach ($section['fields'] AS $field_id => $field) {
        $key = drupal_strtolower($section_id . '.' . $field_id);
        $minimum_requirements[ $key ] = $field['minimum_requirements'];
      }

      if (isset($section['minimum_requirements'])) {
        $minimum_requirements[ $section_id ] = $section['minimum_requirements'];
      }
    }
  }

  if (!empty($field_name)) {
    $field_name = drupal_strtolower($field_name);
    $field_id = preg_replace("/\_/", ".", $field_name);

    return $minimum_requirements[$field_id];
  }

  return $minimum_requirements;
}

/**
 * Get configured minimum requirements of a field.
 *
 * @param string $field_name
 *   The name  or id of field to check.
 *
 * @return integer
 *   The configured requrements level of this field.
 *   ARCHIBALD_RES_FIELD_OPTIONAL, ARCHIBALD_RES_FIELD_RECOMMEND or ARCHIBALD_RES_FIELD_REQUIRED
 */
function archibald_get_field_requirements($field_name) {
  $field_name = drupal_strtolower($field_name);
  $field_id = str_replace('_', '.', $field_name);
  $field_name = str_replace('.', '_', $field_name);

  $minimum_requirements = archibald_get_field_minimum_requirements($field_id);
  $requirements = variable_get('archibald_field_require_type_' . $field_name, $minimum_requirements);

  if ($minimum_requirements > $requirements && $minimum_requirements >= ARCHIBALD_RES_FIELD_REQUIRED) {
    // If minimum value is REQIRED the current value can not below
    // But if it only is RECOMMEND the current value can be below.
    return $minimum_requirements;
  }

  return $requirements;
}

/**
 * This function returns the translated label for a given field name.
 *
 * @param string $field_name
 *   The LOM metadata field name
 *
 * @param bollean $full_path
 *   Set this value to TRUE to get a full name path.
 *   Default: FALSE
 *
 * @return string
 *   The Label of the Field
 */
function archibald_get_field_label($field_name, $full_path = FALSE) {

  $field_info = archibald_get_field_info();

  $field_name = preg_replace("/\_/", ".", $field_name);
  $parts = explode('.', $field_name);
  $count = count($parts);
  $label = '';

  if (isset($parts[2])) {
    if ($parts[2] == 'value') {
      unset($parts[2]);
    }
  }

  switch ($count) {
    case 1:
      $label = $field_info[$parts[0]]['label'];
      break;
    case 2:
      if ($full_path == TRUE) {
        $label = $field_info[$parts[0]]['label'];
        $label .= ' > ' . $field_info[$parts[0]]['fields'][$parts[1]]['label'];
      }
      else {
        $label .= $field_info[$parts[0]]['fields'][$parts[1]]['label'];
      }
      break;
    case 3:
      if ($full_path == TRUE) {
        $label = $field_info[$parts[0]]['label'];
        $label .= ' > ' . $field_info[$parts[0]]['fields'][$parts[1]]['label'];
        $label .= ' > ' . $field_info[$parts[0]]['fields'][$parts[1]]['subfields'][$parts[2]]['label'];
      }
      else {
        $label = $field_info[$parts[0]]['fields'][$parts[1]]['subfields'][$parts[2]]['label'];
      }

      break;
  }

  if (empty($label)) {
    $label = preg_replace("/\./", " ", $field_name);
  }

  return $label;
}